#début du code

#0. Importation des données (Excel Reader)
#individus de la feuille "Ind.Actifs"

#1. correlation plot (R View Table)

#variables actives
#voir le rôle de la variable prédéfinie knime.in
df <- knime.in
df <- df[,colnames(df)!="Modele"]

#calcul de la corrélation
mcor <- cor(df)

#affichage du corrélation plot
#package doit être installé dans R au préalable
library(corrplot)
corrplot::corrplot(mcor)

#affichage de la matrice dans la sortie standard (stdout)
print(mcor)

#1' -- voir les outils de visualisation et d'exportation d'image
#à la sortie de R View (Table)

# Image View
# Image Writer (Port)

#2. ACP avec le package "psych" (R Learner)
#parce que fait la rotation varimax

#récupérer les données
D <- knime.in

#isoler les variables actives
DAct <- D[,colnames(D)!='Modele']

#acp avec psych (à installer au préalable aussi dans R)
#psych parce que propose la rotation varimax
library(psych)

#acp
mon_acp <- psych::principal(DAct,nfactors=2,scores=TRUE,rotate="varimax")

#envoyer des infos dans la sortie stdout
print(mon_acp)

#et en réalité, l'espace mémoire (workspace)
#est envoyé en sortie du composant
#il contient différents objets -- à voir

#3. Eboulis des valeurs propres [R View (Worspace)]
#voir dans "workspace" les objets disponibles

#puis plot de l'éboulis des v.p.
plot(mon_acp$values,type="b",main="Eboulis des v.p.")

#4. Cercle des corrélations from scratch [R View (Worspace)]
#récupération de l'objet ACP dans le workspace

#récupérer les corrélations
c1 <- mon_acp$loadings[,1]
c2 <- mon_acp$loadings[,2]

#l'option "asp" permet d'assurer l'égalité des échelles
#en abscisse et en ordonnée
plot(c1,c2,xlim=c(-1,+1),ylim=c(-1,+1),type="n",asp=1,main="Cercle des correlations")
abline(h=0,v=0)
text(ifelse(c1>0,c1+0.1,c1-0.1),jitter(c2,100),labels = colnames(DAct), cex=0.75,col="blue")
symbols(0,0,circles=1,inches=F,add=T)
#mettre les flèches
arrows(0,0,x1=c1,y1=c2,angle=10,col="gray")

#5. Projection des individus [R View (Worspace)]

#projections des individus
#récupérer les coordonnées
f1 <- mon_acp$scores[,1]
f2 <- mon_acp$scores[,2]

#affichage -- attention aux dimensions des repères abscisse vs. ordonnée
plot(f1,f2,main="Plan factoriel",type="n",,xlim=c(-2.5,+2.5),ylim=c(-2.5,+2.5),asp=1)
abline(h=0,v=0)
text(f1,f2,D$Modele,cex=0.65)

#6. K-Means sur facteurs (R to R)
#R to R parce qu'en sortie on veut avoir le worskpace
#avec le nouvel objet K-Means

#k-means sur facteurs
#librairie "fm2id" parce que propose la fonction predict
library(fdm2id)
mon_km <- fdm2id::KMEANS(mon_acp$scores,k=4)

#effectifs des groupes
print(table(mon_km$cluster))

#7. Projection des individus avec groupes d'appartenance [R View (Worspace)]

#récupérer les coordonnées
f1 <- mon_acp$scores[,1]
f2 <- mon_acp$scores[,2]

#graphique
plot(f1,f2,main="Plan factoriel",type="n",xlim=c(-2.5,+2.5),ylim=c(-2.5,+2.5),asp=1)
abline(h=0,v=0)
text(f1,f2,D$Modele,cex=0.65,col=c('red4','green4','blue','black')[mon_km$cluster])

#8. Importation des individus supplémentaires (Excel Reader)
#les individus de la seconde feuille - Voitures Peugeot

#9. Traitement des individus supplémentaires (R Predictor)
#2 étapes : (1) calcul des coordonnées factorielles
# (2) Affectation des individus aux groupes
# /!\ de l'intérêt des fonctions "predict"

#chargement des individus supplémentaires
DBis <- knime.in

#isoler les variables concernées
DSupp <- DBis[,colnames(DBis)!='Modele']

#calcul des coordoonées factorielles -> predict
fSupp <- predict(mon_acp,DSupp)

#rattachement au cluster d'appartenance -> predict
kSupp <- predict(mon_km,fSupp)

#xportation - knime.out est une variable prédéfinie
knime.out <- as.data.frame(fSupp)
knime.out["Modele"] <- DBis$Modele
knime.out["Cluster"] <- kSupp

#10. Fusion dans le même workspace (Add Table To R)
#du workspace des individus actifs
#et des infos sur les données supplémentaires

#infos sur les individus supplémentaires
#récupéré dans un data.frame
dfSupp <- knime.in

#affichage des objets dispos (stdout)
print(ls())

#11. Projection avec les individus supp. [R View (Worspace)]

#récupérer les coordonnées des individus actifs
f1 <- mon_acp$scores[,1]
f2 <- mon_acp$scores[,2]

#prépa. affichage
plot(f1,f2,main="Plan factoriel",type="n",xlim=c(-2.5,+2.5),ylim=c(-2.5,+2.5),asp=1)
abline(h=0,v=0)
#ind. actifs
text(f1,f2,D$Modele,cex=0.65,col=c('coral','chartreuse','deepskyblue','gray')[mon_km$cluster])
#ind. illustratifs
text(dfSupp$RC1,dfSupp$RC2,dfSupp$Modele,cex=0.8,col=c('red4','green4','blue','black')[dfSupp$Cluster])

